import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { json, max, select, geoPath, geoNaturalEarth1, scaleSequential, interpolateYlOrRd } from 'd3';
import { feature } from 'topojson';
import { useData } from './useData';

const jsonUrl = "https://cdn.jsdelivr.net/npm/world-atlas@2/countries-110m.json";
const dataUrl = "https://gist.githubusercontent.com/bajisharif-2902/8c6cc76cf7e64e29544e379f47a3ddcd/raw/0e39faefe47eebb6b6c4a31e97f6299ec77d6fef/nuclear(1945-2022).csv";
const countryUrl = "https://gist.githubusercontent.com/bajisharif-2902/853d4711ddf3a72770464e179e69e4f6/raw/1a54379335147f8bbf70b96f8b19d3048a8177df/countrycode.csv";

const width = 900;
const height = 450;
const selectedYear = 2019;

const App = () => {
  const [atlasData, setAtlasData] = useState(null);
  const data = useData(dataUrl);
  const codes = useData(countryUrl);
  
  useEffect(() => {
  	json(jsonUrl).then(topojsonData => {
      const { countries } = topojsonData.objects;
      setAtlasData(feature(topojsonData, countries));
    });
  }, []);
  
  if (!atlasData || !data || !codes) {
    return <div>Loading...</div>;
  }
  
	const codeMap = new Map();
  codes.forEach(code => codeMap.set(code['country-code'], code['alpha-3']));
  
  const newData = data.filter(d => +d.Year === selectedYear);
  const dataMap = new Map();
  newData.forEach(country => dataMap.set(country.Code, country));
  
  const colorValue = d => d['Deaths - Brain and central nervous system cancer - Sex: Both - Age: All Ages (Number)'];
  const colorScale = scaleSequential(interpolateYlOrRd).domain([0, max(newData, colorValue)]);
  
  const maxValue = +max(newData, colorValue);
  let legendDomain = [];
  for(let i=0; i <= maxValue; i += parseInt(maxValue/5)) {
   	legendDomain.push(i); 
  }
  console.log(legendDomain);
  
  const projection = geoNaturalEarth1().scale(width / 1.3 / Math.PI).translate([width/2, height/2]);
  const path = geoPath(projection);
	return (
  	<>
      <svg width={width} height={height} style={{border: '1px solid black'}}>
        <g>
          {
          	atlasData.features.map(feature => {
              						const code = codeMap.get(feature.id);
              						const country = dataMap.get(code);
                          return (<g onMouseEnter={() => {
                                if (country) {
                              		select('#map-toolkit').append('span').text(country.Entity + ' - Total Deaths: ' + colorValue(country));
                                }
                              }} onMouseLeave={() => {
                              	if (country) {
                                	select('#map-toolkit').selectAll('span').remove();
                                }
                              }}>
                                   <path d={path(feature)} fill={country ? colorScale(colorValue(country)) : ''} style={{stroke: '#fff'}} >
                                     <text>Hello</text>
                                     </path>
          												</g>);
            }
            )
         	}
        </g>
        <g transform={`translate(20, ${height/2})`} >
          {
          	legendDomain.map((d,i) => 
                             <>
                             <rect y={i*20} width={30} height={15} fill={colorScale(d)} />
                             </>   
                            )
          }
        </g>
      </svg>
      <div id="map-toolkit" style={{ top: height - 15, width: width }}></div>
    </>
  );
}

ReactDOM.render(<App />, document.getElementById('root'));
